#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <vector>

#include "ntfs/NTFSParser.h"
#include "fat/FATParser.h"
#include "filesystem/Directory.h"
#include "MBRParser.h"

using std::vector;

void interpretMBR(FILE* device) {
  MBRParser* mbr = new MBRParser(device, 0);

  vector<Directory*> roots = mbr->getRoots();
  for(unsigned int i = 0; i < roots.size(); i++) {
    roots[i]->print();
  }

  delete mbr;
}

void printDirectory(Directory* dir) {
	if(dir != NULL) {
		vector<File*> files = dir->getFiles();
		vector<Directory*> dirs = dir->getDirectories();
		
		dir->print();
		printf("\n");
		for(unsigned int i = 0; i < files.size(); i++) {
			files[i]->print();
		}
		
		for(unsigned int i = 0; i < dirs.size(); i++) {
			printDirectory(dirs[i]);
		}
	}
}

void printFileListing(FilesystemParser* parser) {
  Directory* root = parser->getRoot();
  printDirectory(root);
	delete root;
  delete parser;
}

void printArguments(char* appName) {
  printf("Invalid command line parameters: %s <Filesystem {RAW, NTFS, FAT}> <device or image file>\n", appName);
  exit(1);
}

int main(int argc, char** argv) {
  if(argc != 3) {
    printArguments(argv[0]);
  }

  FILE* device = fopen(argv[2], "r");

  if(device == NULL) {
    printf("Invalid device: %s\n", argv[2]);
    return 1;
  }

  char* filesystem = argv[1];
  if(!strcmp(filesystem, "RAW")) {
    interpretMBR(device);
  }
  else if(!strcmp(filesystem, "NTFS")) {
    printFileListing(new NTFSParser(device, 0));
  }
  else if(!strcmp(filesystem, "FAT")) {
    printFileListing(new FATParser(device, 0));
  }
  else {
    printArguments(argv[0]);
  }

  fclose(device);

  return 0;
}

