#ifndef NTFS_ATTRIBUTE_H
#define NTFS_ATTRIBUTE_H

#include "../common.h"
#include "../DeviceInformation.h"

#include <stdio.h>

#include "Data.h"
#include "DataRun.h"
#include <vector>

enum NTFSAttributeType {
    STANDARD_INFORMATION = 1,
    ATTRIBUTE_LIST,
    FILENAME,
    OBJECT_ID,
    SECURITY_DESCRIPTOR,
    VOLUME_NAME,
    VOLUME_INFORMATION,
    DATA,
    INDEX_ROOT,
    INDEX_ALLOCATION,
    BITMAP,
    REPARSE_POINT,
    EA_INFORMATION,
    EA,
    PROPERTY_SET,
    LOGGED_UTILIITY_STREAM,
    INVALID
};

using std::vector;

class NTFSAttribute
{
public:
    NTFSAttribute(DeviceInformation* device, byte* attributeStart, int mftRecordOffset);
    virtual ~NTFSAttribute();

    void setType(int intType);
    virtual void print();
    bool isValid();
    int getLength();
    NTFSData getData();
    bool getResident();
    NTFSAttributeType getType();
    bool containsDataRuns();

protected:
    int getHeaderLength();
    int flags;
    bool hasName();
    const char* getName();
    int getNameLength();
    vector<NTFSDataRun*> getDataRuns();
		
private:
    NTFSAttributeType type;
    int length;
    int nonResident;
    int lengthOfName;
    int offsetToName;
    int attributeID;
    int sizeOfContent;
    int offsetToContent;
    char* name;
    bool valid;
    DeviceInformation* device;

    vector<NTFSDataRun*> dataRuns;

};


NTFSAttribute* getNTFSAttribute(DeviceInformation* device, byte* offset, int mftRecordOffset);

#endif

