#include "IndexAllocationAttribute.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "IndexEntry.h"

NTFSIndexAllocationAttribute::NTFSIndexAllocationAttribute(DeviceInformation* device, byte* offset, int mftRecordOffset)
  : NTFSAttribute(device, offset, mftRecordOffset) {

  // get the directory index data
  NTFSData ntfsdata = this->getData();
  
  unsigned char* data = ntfsdata.getData();
  int offsetToIndexEntry = 0;

  // get the magic number and do a basic sanity check
  char magicNumber[5];
  for(int i = 0; i < 4; i++) {
    magicNumber[i] = data[i];
  }
  magicNumber[4] = 0;

  if(strcmp(magicNumber, "INDX") == 0) {
		//printf("%d\n", data[9]);
		offsetToIndexEntry = 0x18 + get_dword(&data[0x18]);

		while(true) {
			IndexEntry entry = IndexEntry(&data[offsetToIndexEntry]);
			if(entry.isLast()) {
				break;
			}
			
			entries.push_back(entry);
			offsetToIndexEntry += entry.getLength();
		}
  }
	else {
		fprintf(stderr, "ERROR: Magic Index record not found\n");
	}

	free(data);
}

vector<IndexEntry> NTFSIndexAllocationAttribute::getEntries() {
	return entries;
}

void NTFSIndexAllocationAttribute::print() {
  printf("Index allocation attribute: ");
  if(hasName()) {
    printf(" (%s) ", getName());
  }
  printf("\n");
}
