#include "mftentry.h"
#include <stdio.h>
#include "FilenameAttribute.h"
#include "IndexRootAttribute.h"
#include "IndexAllocationAttribute.h"
#include <string.h>

MFTEntry::MFTEntry(DeviceInformation* device, unsigned char* mft) {
    for(int i = 0; i < 4; i++) {
      magicMarker[i] = mft[i];
    }
    magicMarker[4] = 0;
		
    updateSequence1         = get_word(&mft[0x04]);
    numEntriesInFixupArray  = get_word(&mft[0x06]);
    sequenceNumber          = get_longlong(&mft[0x08]);
    hardLinkCount           = get_word(&mft[0x12]);
    firstAttribute          = get_word(&mft[0x14]);
    flags                   = get_word(&mft[0x16]);
    usedSizeOfMFTEntry      = get_dword(&mft[0x18]);
    allocatedSizeOfMFTEntry = get_dword(&mft[0x1c]);
    fileReference           = get_longlong(&mft[0x20]);
    fileRecordNumber        = get_6bytes(&mft[0x20]);
    fileSequenceNumber      = get_word(&mft[0x26]);
    nextAttributeID         = get_word(&mft[0x28]);
    numberOfEntry           = get_word(&mft[0x2C]);
    
    if(flags >= 0x04) {
        //printf("Record not in use (%d)\n", flags);
    }
    else if(flags == 0x03) {
        //printf("Type: Directory\n");
    }
    else {
        //printf("Type: File\n");
    }
    
    // run through the attributes, and add them to the list...
    int offset = firstAttribute;
	do {

      NTFSAttribute* currAttr = getNTFSAttribute(device, &mft[offset], offset);
      if(!currAttr->isValid() || currAttr->getType() == INVALID) {
				delete currAttr;
        break;
      }
      else {
        attributes.push_back(currAttr);
      }
 
      offset = offset + currAttr->getLength();

    } while(offset <= usedSizeOfMFTEntry);
		
    //print();
}

MFTEntry::~MFTEntry() {
  for(unsigned int i = 0; i < attributes.size(); i++) {
    delete attributes[i];
  }
}

NTFSData MFTEntry::getData() {
  for(unsigned int i = 0; i < attributes.size(); i++) {
    NTFSAttributeType type = attributes[i]->getType();

    if(type == DATA && attributes[i]->getResident() == false) {
      return attributes[i]->getData();
    }
  }

  return NTFSData(0);
}

bool MFTEntry::isDirectory() {
	return (flags & 0x03) == 0x03;
}

const char* MFTEntry::getFilename() {
  for(unsigned int i = 0; i < attributes.size(); i++) {
    if(attributes[i]->getType() == FILENAME) {
      return ((NTFSFilenameAttribute*)attributes[i])->getRawFilename();
    }
  }
  return NULL;
}

// The following method smells really bad - but it implemented this way for performance reasons.
vector<IndexEntry> MFTEntry::getIndexEntries() {
	vector<IndexEntry> ret;
	
	for(unsigned int i = 0; i < attributes.size(); i++) {
		int type = attributes[i]->getType();
		
		if(type == INDEX_ROOT) {
			vector<IndexEntry> entries = ((NTFSIndexRootAttribute*)attributes[i])->getEntries();
			for(unsigned int i = 0; i < entries.size(); i++) {
				entries[i].print();
				ret.push_back(entries[i]);
			}
		}
		else if(type == INDEX_ALLOCATION) {
			vector<IndexEntry> entries = ((NTFSIndexAllocationAttribute*)attributes[i])->getEntries();
			for(unsigned int i = 0; i < entries.size(); i++) {
				entries[i].print();
				ret.push_back(entries[i]);
			}
		}
	}
	
	return ret;
}

void MFTEntry::print() {
  if(flags != 0x03)
    return;
	
  for(unsigned int i = 0; i < attributes.size(); i++) {
    attributes[i]->print();
  }
   
  for(unsigned int i = 0; i < attributes.size(); i++) {
    if(attributes[i]->getType() == DATA) {
      attributes[i]->getData();
    }
  }

  printf("\n\n");
}

void MFTEntry::printDirectories() {
  /*if(flags != 0x03)
    return;

  NTFSFilenameAttribute* filename = getFilename();
  if(filename) {
    filename->print();
    printf("\n");
  }*/
}

