/*
 * =====================================================================================
 *
 *       Filename:  hexView.cpp
 *
 *    Description:  A simple hex viewer to be used while debugging the NTFS code.
 *
 *        Version:  1.0
 *        Created:  05/08/09 16:53:01
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Benjamin Kearns (benjamin.kearns@gmail.com)
 *
 * =====================================================================================
 */


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

void printBytes(unsigned char* bytes) {
  for(int i = 0; i < 10; i++) {
    char hex[3];
    snprintf(hex, 3, "%X", bytes[i]);
    int length = strlen(hex);

    if(length == 0) {
      printf("00 ");
    }
    else if(length == 1) {
      printf("0%s ", hex);
    }
    else {
      printf("%s ", hex);
    }
  }
}

void hexDump(unsigned char* bytes, unsigned long length) {
  for(unsigned long i = 0; i < length / 20; i++) {
    printBytes(&bytes[i*20]);
    printf("  ");
    printBytes(&bytes[(i*20) + 10]);
    printf("   | ");
    
    for(int j = 0; j < 20; j++) {
      unsigned char ascii = bytes[(i*20) + j];

      if(ascii < 32 || ascii > 127) {
        ascii = '.';
      }
      printf("%c", ascii);
    }

    printf("\n");
  }
}

void printUsage(char* progName) {
  printf("Usage: %s <device> <length> <lcn offset>\n", progName);
  exit(EXIT_FAILURE);
}

int main(int argc, char *argv[]) {
  // check that the right number of command line parameters have been passed 
  if (argc != 4) {
    printUsage(argv[0]);
  }

  long length = atol(argv[2]) * 512;
  long offset = atol(argv[3]);

  // open the device for reading...
  FILE	*f;										// input-file pointer
  char	*f_file_name = argv[1];         		// input-file name

  f	= fopen( f_file_name, "r" );
  if(f == NULL) {
    fprintf(stderr, "Could not open file '%s'\n", f_file_name);
    exit(EXIT_FAILURE);
  }

  // allocate the space for the bytes to read, and actually read the data in
  unsigned char* data = (unsigned char*)malloc(length);
  if(data==NULL) {
    fprintf(stderr, "\nDynamic memory allocation failed\n");
    exit(EXIT_FAILURE);
  }

  printf("Seeking to: %ld\n", offset*512);
  fseeko(f, offset*512, SEEK_SET);
  fread(data, 1, length, f);

  // now pass this on to the hex dumping function...
  hexDump(data, length);

  // clean up
  free(data);
  data = NULL;

  if(fclose(f) == EOF ) {			// close input file
    fprintf(stderr, "Could not close file '%s'\n", f_file_name);
    exit(EXIT_FAILURE);
  }

  return EXIT_SUCCESS;
}

